@@ -1,4 +1,5 @@
 /* This file is part of Spectacle, the KDE screenshot utility
+ * Copyright 2019 David Redondo <kde@david-redondo.de>
  * Copyright (C) 2015 Boudhayan Gupta <bgupta@kde.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,6 +22,9 @@
 
 #include "ExportManager.h"
 
+#include "settings.h"
+#include <kio_version.h>
+
 #include <QDir>
 #include <QMimeData>
 #include <QMimeDatabase>
@@ -34,6 +38,7 @@
 #include <QRegularExpression>
 #include <QRegularExpressionMatch>
 #include <QString>
+#include <QRandomGenerator>
 
 #include <KSharedConfig>
 #include <KIO/ListJob>
@@ -41,7 +46,7 @@
 #include <KIO/FileCopyJob>
 #include <KIO/StatJob>
 
-#include "SpectacleConfig.h"
+
 
 ExportManager::ExportManager(QObject *parent) :
     QObject(parent),
@@ -49,9 +54,7 @@ ExportManager::ExportManager(QObject *parent) :
     mTempFile(QUrl()),
     mTempDir(nullptr)
 {
-    connect(this, &ExportManager::imageSaved, [](const QUrl &savedAt) {
-        SpectacleConfig::instance()->setLastSaveFile(savedAt);
-    });
+    connect(this, &ExportManager::imageSaved, &Settings::setLastSaveLocation);
 }
 
 ExportManager::~ExportManager()
@@ -119,7 +122,8 @@ void ExportManager::setTimestamp(const QDateTime &timestamp)
 
 QString ExportManager::defaultSaveLocation() const
 {
-    QString savePath = SpectacleConfig::instance()->defaultSaveLocation().toLocalFile();
+    const QUrl saveUrl = Settings::self()->defaultSaveLocation();
+    QString savePath = saveUrl.scheme().isEmpty() ? saveUrl.toString() : saveUrl.toLocalFile();
     savePath = QDir::cleanPath(savePath);
 
     QDir savePathDir(savePath);
@@ -136,7 +140,7 @@ QUrl ExportManager::getAutosaveFilename()
     const QDir baseDirPath(baseDir);
     const QString filename = makeAutosaveFilename();
     const QString fullpath = autoIncrementFilename(baseDirPath.filePath(filename),
-                                                   SpectacleConfig::instance()->saveImageFormat(),
+                                                   Settings::self()->defaultSaveImageFormat().toLower(),
                                                    &ExportManager::isFileExists);
 
     const QUrl fileNameUrl = QUrl::fromUserInput(fullpath);
@@ -160,7 +164,7 @@ QString ExportManager::truncatedFilename(QString const &filename)
 
 QString ExportManager::makeAutosaveFilename()
 {
-    return formatFilename(SpectacleConfig::instance()->autoSaveFilenameFormat());
+    return formatFilename(Settings::self()->saveFilenameFormat());
 }
 
 QString ExportManager::formatFilename(const QString &nameTemplate)
@@ -173,7 +177,7 @@ QString ExportManager::formatFilename(const QString &nameTemplate)
     if (mCaptureMode == Spectacle::CaptureMode::ActiveWindow ||
         mCaptureMode == Spectacle::CaptureMode::TransientWithParent ||
         mCaptureMode == Spectacle::CaptureMode::WindowUnderCursor) {
-        title = mWindowTitle.replace(QLatin1String("/"), QLatin1String("_"));  // POSIX doesn't allow "/" in filenames
+        title = mWindowTitle.replace(QLatin1Char('/'), QLatin1String("_"));  // POSIX doesn't allow "/" in filenames
     } else {
         // Remove '%T' with separators around it
         const auto wordSymbol = QStringLiteral(R"(\p{L}\p{M}\p{N})");
@@ -190,12 +194,12 @@ QString ExportManager::formatFilename(const QString &nameTemplate)
                              .replace(QLatin1String("%m"), timestamp.toString(QStringLiteral("mm")))
                              .replace(QLatin1String("%S"), timestamp.toString(QStringLiteral("ss")))
                              .replace(QLatin1String("%T"), title);
-                             
+
     // check if basename includes %[N]d token for sequential file numbering
     QRegularExpression paddingRE;
     paddingRE.setPattern(QStringLiteral("%(\\d*)d"));
     QRegularExpressionMatch paddingMatch;
-                             
+
     while (result.indexOf(paddingRE, 0, &paddingMatch) > -1) {
         int highestFileNumber = 0;
 
@@ -208,13 +212,14 @@ QString ExportManager::formatFilename(const QString &nameTemplate)
         // search save directory for files
         QDir dir(baseDir);
         const QStringList fileNames = dir.entryList(QDir::Files, QDir::Name);
-        
+
         // if there are files in the directory...
         if (fileNames.length() > 0) { 
-            QString resultCopy = result;
+            QString resultCopy = QRegularExpression::escape(result);
             QRegularExpression fileNumberRE;
             const QString replacement = QStringLiteral("(\\d{").append(QString::number(paddedLength)).append(QLatin1String(",})"));
-            const QString fullNameMatch = QStringLiteral("^").append(resultCopy.replace(paddingMatch.captured(),replacement)).append(QStringLiteral("\\..*$"));
+            QString escapedMatch = QRegularExpression::escape(paddingMatch.captured());
+            const QString fullNameMatch = QStringLiteral("^").append(resultCopy.replace(escapedMatch,replacement)).append(QStringLiteral("\\..*$"));
             fileNumberRE.setPattern(fullNameMatch);
 
             // ... check the file names for string matching token with padding specified in result
@@ -233,7 +238,7 @@ QString ExportManager::formatFilename(const QString &nameTemplate)
         }
         // replace placeholder with next number padded
         const QString nextFileNumberPadded = QString::number(highestFileNumber + 1).rightJustified(paddedLength, QLatin1Char('0'));
-        result = result.replace(paddingMatch.captured(), nextFileNumberPadded);
+        result.replace(paddingMatch.captured(), nextFileNumberPadded);
     }
 
     // Remove leading and trailing '/'
@@ -245,7 +250,7 @@ QString ExportManager::formatFilename(const QString &nameTemplate)
     }
 
     if (result.isEmpty()) {
-        result = SpectacleConfig::instance()->defaultFilename();
+        result = QStringLiteral("Screenshot");
     }
     return truncatedFilename(result);
 }
@@ -253,7 +258,7 @@ QString ExportManager::formatFilename(const QString &nameTemplate)
 QString ExportManager::autoIncrementFilename(const QString &baseName, const QString &extension,
                                              FileNameAlreadyUsedCheck isFileNameUsed)
 {
-    QString result = truncatedFilename(baseName) + QLatin1Literal(".") + extension;
+    QString result = truncatedFilename(baseName) + QLatin1String(".") + extension;
     if (!((this->*isFileNameUsed)(QUrl::fromUserInput(result)))) {
         return result;
     }
@@ -269,7 +274,7 @@ QString ExportManager::autoIncrementFilename(const QString &baseName, const QStr
     // unlikely this will ever happen, but just in case we've run
     // out of numbers
 
-    result = fileNameFmt.arg(QStringLiteral("OVERFLOW-") + QString::number(qrand() % 10000));
+    result = fileNameFmt.arg(QLatin1String("OVERFLOW-") + QString::number(QRandomGenerator::global()->bounded(10000)));
     return truncatedFilename(result) + extension;
 }
 
@@ -279,7 +284,7 @@ QString ExportManager::makeSaveMimetype(const QUrl &url)
     QString type = mimedb.mimeTypeForUrl(url).preferredSuffix();
 
     if (type.isEmpty()) {
-        return SpectacleConfig::instance()->saveImageFormat();
+        return Settings::self()->defaultSaveImageFormat().toLower();
     }
     return type;
 }
@@ -287,7 +292,7 @@ QString ExportManager::makeSaveMimetype(const QUrl &url)
 bool ExportManager::writeImage(QIODevice *device, const QByteArray &format)
 {
     QImageWriter imageWriter(device, format);
-    imageWriter.setQuality(SpectacleConfig::instance()->compressionQuality());
+    imageWriter.setQuality(Settings::self()->compressionQuality());
     /** Set compression 50 if the format is png. Otherwise if no compression value is specified
      *  it will fallback to using quality (QTBUG-43618) and produce huge files.
      *  See also qpnghandler.cpp#n1075. The other formats that do compression seem to have it
@@ -429,8 +434,12 @@ bool ExportManager::isFileExists(const QUrl &url) const
     if (!(url.isValid())) {
         return false;
     }
-
+#if KIO_VERSION < QT_VERSION_CHECK(5, 69, 0)
     KIO::StatJob * existsJob = KIO::stat(url, KIO::StatJob::DestinationSide, 0);
+#else
+    KIO::StatJob * existsJob = KIO::statDetails(url, KIO::StatJob::DestinationSide, KIO::StatDetail::Basic);
+#endif
+
     existsJob->exec();
 
     return (existsJob->error() == KJob::NoError);
@@ -454,7 +463,6 @@ void ExportManager::doSave(const QUrl &url, bool notify)
     if (save(savePath)) {
         QDir dir(savePath.path());
         dir.cdUp();
-        SpectacleConfig::instance()->setLastSaveFile(savePath);
 
         emit imageSaved(savePath);
         if (notify) {
@@ -466,7 +474,6 @@ void ExportManager::doSave(const QUrl &url, bool notify)
 bool ExportManager::doSaveAs(QWidget *parentWindow, bool notify)
 {
     QStringList supportedFilters;
-    SpectacleConfig *config = SpectacleConfig::instance();
 
     // construct the supported mimetype list
     const auto mimeTypes = QImageWriter::supportedMimeTypes();
@@ -475,12 +482,12 @@ bool ExportManager::doSaveAs(QWidget *parentWindow, bool notify)
     }
 
     // construct the file name
-    const QString filenameExtension = SpectacleConfig::instance()->saveImageFormat();
+    const QString filenameExtension = Settings::self()->defaultSaveImageFormat().toLower();
     const QString mimetype = QMimeDatabase().mimeTypeForFile(QStringLiteral("~/fakefile.") + filenameExtension, QMimeDatabase::MatchExtension).name();
     QFileDialog dialog(parentWindow);
     dialog.setAcceptMode(QFileDialog::AcceptSave);
     dialog.setFileMode(QFileDialog::AnyFile);
-    dialog.setDirectoryUrl(config->lastSaveAsLocation());
+    dialog.setDirectoryUrl(Settings::self()->lastSaveAsLocation().adjusted(QUrl::RemoveFilename));
     dialog.selectFile(makeAutosaveFilename() + QStringLiteral(".") + filenameExtension);
     dialog.setDefaultSuffix(QStringLiteral(".") + filenameExtension);
     dialog.setMimeTypeFilters(supportedFilters);
@@ -492,8 +499,7 @@ bool ExportManager::doSaveAs(QWidget *parentWindow, bool notify)
         if (saveUrl.isValid()) {
             if (save(saveUrl)) {
                 emit imageSaved(saveUrl);
-                config->setLastSaveAsFile(saveUrl);
-
+                Settings::setLastSaveAsLocation(saveUrl);
                 if (notify) {
                     emit forceNotify(saveUrl);
                 }
@@ -504,15 +510,32 @@ bool ExportManager::doSaveAs(QWidget *parentWindow, bool notify)
     return false;
 }
 
-// misc helpers
+void ExportManager::doSaveAndCopy(const QUrl &url)
+{
+    if (mSavePixmap.isNull()) {
+        emit errorMessage(i18n("Cannot save an empty screenshot image."));
+        return;
+    }
 
+    QUrl savePath = url.isValid() ? url : getAutosaveFilename();
+    if (save(savePath)) {
+        QDir dir(savePath.path());
+        dir.cdUp();
+        Settings::setLastSaveLocation(savePath);
+
+        doCopyToClipboard(false);
+        emit imageSavedAndCopied(savePath);
+    }
+}
+
+// misc helpers
 void ExportManager::doCopyToClipboard(bool notify)
 {
     auto data = new QMimeData();
     data->setImageData(mSavePixmap.toImage());
     data->setData(QStringLiteral("x-kde-force-image-copy"), QByteArray());
     QApplication::clipboard()->setMimeData(data, QClipboard::Clipboard);
-
+    emit imageCopied();
     if (notify) {
         emit forceNotify(QUrl());
     }
